SystemOrganization addCategory: #'Pier-ScriptingAPI'!
SystemOrganization addCategory: #'Pier-ScriptingAPI-Scalability'!

!SequenceableCollection methodsFor: '*pier-scriptingapi' stamp: 'kph 9/15/2006 15:43'!
joinTokens: aString
    ^ String streamContents:
        [:s |
        self do: [:ea | s nextPutAll: ea asString] separatedBy: [s nextPutAll: aString]] ! !

!SequenceableCollection methodsFor: '*pier-scriptingapi' stamp: 'kph 9/15/2006 00:53'!
join: aCollection

	^ self class streamContents: [ :stream |
		aCollection
			do: [ :each | stream nextPut: each ]
			separatedBy: [ stream nextPutAll: self ] ]! !

!Collection methodsFor: '*pier-scriptingapi' stamp: 'kph 9/15/2006 00:36'!
joinWith: separator
	
^separator join: self ! !

!Character methodsFor: '*pier-scriptingapi' stamp: 'kph 9/15/2006 15:39'!
join: aCollection

	^ String streamContents: [ :stream |
		aCollection
			do: [ :each | stream nextPutAll: each printString ]
			separatedBy: [ stream nextPut: self ] ]! !

Object subclass: #PRModelScripter
	instanceVariableNames: 'kernel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI'!

!PRModelScripter commentStamp: '<historical>' prior: 0!
Implementing the same protocol as PRCommandScripter. PRModelScripter is simple programmatic interface to Pier which operates on the model directly. !

PRModelScripter subclass: #PRCommandScripter
	instanceVariableNames: 'context command logging'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI'!

!PRCommandScripter commentStamp: '<historical>' prior: 0!
PRCommandScripter a simple programmatic interface to Pier, which uses the PRCommand mechanism to interface with the model. 

This means that all actions may be logged as if made through the Seaside interface. 

Logging can be turned on/off with #noLog #yesLog. If logging is disabled then the PRCommandScripter should be equivalent to PRModelScriptor but with an implementation more consistent with the end user experience. PRModelScriptor provides examples of how the Model may be used but PRCommandScriptor is the preferred interface to command functions. 
!

PRCommandScripter subclass: #PRCommandAppScripter
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI'!

!PRCommandAppScripter commentStamp: '<historical>' prior: 0!
PRCommandAppScripter provides a scripting interface which executes its commands in the context of a registered seaside application.

The aim of this interface is to ensure that 'WACurrentSession value' returns a useful result when running scripts. PRMagmaPersistency uses 'WACurrentSession value', and command actions should result in the persistent store being updated just as when executed through the seaside user interface.

usage:

PRCommandAppScripter onKernelNamed: 'pier'  
searches for the first application whose configuration entry #kernelName matches the requested name. A session is created using the configuration settings of this application. 

caveat: if you have more than one application serving the same pier kernel with different preferences you have been warned. If no application is found, it will fallback to use any application that is serving all kernels (#mainClass is set to PRMainAllKernels). If you have more than one of these serving more than one repository then it will not check to ensure that it has the correct repository.

PRCommandAppScripter onApp: 'st'
explicitly gives entry point for the application in whose context the scripts are to be run. It is assumed that the value of #kernelName is valid. For an application using PRMainAllKernels, use #onApp:onKernelName: to explicitly target the required kernel.

!

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:04'!
findAppServingAllKernels

^WADispatcher default allApplications detect: [ :app |
					(app configuration valueAt: #mainClass) respondsTo: #isServingAllKernels ]
			 ! !

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:04'!
findAppServingKernel: requestedKernelName ifNone: notThereBlock

^WADispatcher default allApplications detect: [ :app |
					(app configuration valueAt: #kernelName) = requestedKernelName ] 
	ifNone: notThereBlock ! !

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 02:46'!
onApp: entryPointName

| application sessionClass kernelName |

application :=  WADispatcher default entryPointAt: entryPointName.
sessionClass := application configuration valueAt: #sessionClass.
kernelName := application configuration valueAt: #kernelName.

^self setKernel: (PRKernel instanceNamed: kernelName);
	 setSession: (sessionClass application: application); 
	yourself
! !

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 15:16'!
onApp: entryPointName kernelNamed: kernelName

| application sessionClass  |

application :=  WADispatcher default entryPointAt: entryPointName.
sessionClass := application configuration valueAt: #sessionClass.

^self setKernel: (PRKernel instanceNamed: kernelName);
	 setSession: (sessionClass application: application); 
	yourself
! !

!PRCommandAppScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:06'!
onKernelNamed: requestedKernelName

"find a registered seaside application serving the desired kernel and make a session using the settings of that application"

| application sessionClass |

application :=  self findAppServingKernel: requestedKernelName 
					ifNone: [ self findAppServingAllKernels ].

sessionClass := application configuration valueAt: #sessionClass.

^self new setKernel: (PRKernel instanceNamed: requestedKernelName);
	 setSession: (sessionClass application: application); 
	yourself
! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 02:53'!
executeCommand

WACurrentSession use: session during: [ super executeCommand ].! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 17:40'!
kernel

^ kernel persistency resolveKernelFor: self . ! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 05:08'!
session

^session! !

!PRCommandAppScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 02:40'!
setSession: aSession

session := aSession! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:44'!
atPage: aPage addPage: aPageName

"returns the new page added"

command := PRAddCommand context: (self contextOn: aPage).
command name: aPageName.
 
self executeCommand.

^command context structure childrenDecoration at: command name
! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:33'!
atPage: aPage addPage: aName contents: someContent

| newPage |

newPage := self atPage: aPage addPage: aName.

command := PREditCommand context: (self contextOn: newPage).
command 	write: someContent
			using: PRPage descriptionContents.

self executeCommand.

^newPage! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/14/2006 18:59'!
contextOn: aStructure 
^ PRContext kernel: self kernel structure: aStructure  ! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:19'!
executeCommand.

self isLogging ifTrue: [ command execute ] ifFalse: [ self executeCommandWithoutLogging ].! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:44'!
executeCommandWithoutLogging

"code stolen from PRCommand and logging is hacked out"

	command isView
		ifTrue: [ command shouldNotImplement ].
	command kernel persistency doing: command critical: [
		command doValidate; doExecute.
		"command isLogged
			ifTrue: [ command doPersistency ]" ].
	command doAnswer! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:24'!
isLogging

^logging ifNil: [ false ]! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:44'!
noLog

logging := false! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:32'!
removePage: aPage
 
command := PRRemoveCommand context: (self contextOn: aPage).
self executeCommand.
! !

!PRCommandScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:44'!
yesLog

logging := true! !

!PRModelScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/14/2006 23:52'!
onKernelNamed: aKernelName
 
^self onKernel: (PRKernel instanceNamed: aKernelName).! !

!PRModelScripter class methodsFor: 'as yet unclassified' stamp: 'kph 9/14/2006 23:52'!
onKernel: aKernel
 
^self new setKernel: aKernel ; yourself! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/14/2006 21:41'!
atPage: aPage addPage: aName

^aPage addChild: (PRPage named: aName)! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:19'!
atPage: aPage addPage: aName contents: someContent

| newPage |

newPage := self atPage: aPage addPage: aName.
newPage contents: someContent.! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:21'!
atPath: aPath addPage: name contents: someContent

 
^ self atPage: (self structureAt: aPath) addPage: name contents: someContent! !

!PRModelScripter methodsFor: 'compatibility' stamp: 'kph 9/15/2006 02:11'!
isLogging

^false! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 17:40'!
kernel

^ kernel! !

!PRModelScripter methodsFor: 'compatibility' stamp: 'kph 9/15/2006 02:11'!
noLog! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/14/2006 23:48'!
removePageAtPath: aPath

^ self removePage: (self structureAt: aPath) ! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 02:14'!
removePage: aPage

aPage isRoot
		ifTrue: [ self error: 'Unable to remove root node.' ].

PRIncomingReferences
		from: self kernel root
		to: (aPage enumerator
			with; all; hidden;
			contents)
		do: [ :each | each target: nil ].

^ aPage remove! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 02:41'!
setKernel: aKernel

kernel := aKernel
! !

!PRModelScripter methodsFor: 'as yet unclassified' stamp: 'kph 9/14/2006 23:42'!
structureAt: aPath

^PRPathLookup start: self kernel root path: aPath.! !

!PRModelScripter methodsFor: 'compatibility' stamp: 'kph 9/15/2006 02:11'!
yesLog! !

Object subclass: #PRPierifyToPage
	instanceVariableNames: 'target kernel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI'!

!PRPierifyToPage commentStamp: '<historical>' prior: 0!
Abstract class of classes designed to load data from a datasource #target: into a pier page
Can be used with any of the Scripter interface classes
See subclasses for usage examples!

PRPierifyToPage subclass: #PRPierifySmalltalkClass
	instanceVariableNames: ''
	classVariableNames: 'Pace'
	poolDictionaries: ''
	category: 'Pier-ScriptingAPI-Scalability'!

!PRPierifySmalltalkClass commentStamp: 'kph 9/15/2006 04:26' prior: 0!
Example of how to use the scripting interface to import data. This class is instanciated with the data source as its target.

(PRPierifySmalltalkClass target: ProtoObject) into: (PRCommandScripter onKernelNamed: 'Squeak' ).
(PRModelScripter onKernelNamed: 'Squeak') removePageAtPath: 'ProtoObject'.

Removing a page from the model may not be as complete as using the Command approach.

for a sizeable database see class methods #example

(PRPierifySmalltalkClass target: ProtoObject) into: (PRCommandScripter onKernelNamed: 'Squeak' ).
(PRCommandAppScripter onKernelNamed: 'Squeak') removePageAtPath: 'ProtoObject'.
!

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 05:07'!
exampleAllClasses

| model count |

count := 0.
model := PRCommandAppScripter onKernelNamed: 'Squeak'.

Transcript show: model session magmaSession location; cr.
model noLog. 

ProtoObject allSubclassesDo: [ :theClass | 

count := count + 1.
Transcript show: count printString, ': ', theClass name; cr.
[
 
 (PRPierifySmalltalkClass target: theClass) into: model.
 (Delay forSeconds: self pace) wait.
] 
on: PRConflictingNameError do: [ :e | Transcript show: theClass name, ' already exists'; cr. ].


].

 
"[ self exampleAllClasses ] forkAt: 15"

! !

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:35'!
exampleProtoObject

| model |

model := PRCommandAppScripter onKernelNamed: 'Squeak'.
model noLog. 

ProtoObject in: [ :theClass | 

Transcript show: theClass name; cr.
(PRPierifySmalltalkClass target: theClass) into: model ].

"self exampleProtoObject"

! !

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:38'!
pace

^Pace ifNil: [ 20 ].! !

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 04:38'!
pace: t

Pace := t! !

!PRPierifySmalltalkClass class methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 01:17'!
target: aClass  

^self new setTarget: aClass ; yourself! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 03:43'!
classDescriptionToWiki: stream

stream nextPutAll: '!!Description'; cr.
stream nextPutAll: target class comment ; cr.
 

! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 00:13'!
classSubclassesToWiki: stream.

target subclassesDo: [ :sc | stream nextPutAll: '- *', sc name, '*'; cr ]
  ! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/14/2006 23:56'!
methodSelectors: methodDictionary toWiki: stream

methodDictionary keysDo: [ :methodSelector | stream nextPutAll: '- *', methodSelector, '*'; cr ]
 
! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 02:15'!
pageContentsOn: stream  

self classDescriptionToWiki: stream.

stream nextPut: $_; cr.

stream nextPutAll: '!!!!!!Subclasses'; cr.

self classSubclassesToWiki: stream.

stream nextPut: $_; cr.

stream nextPutAll: '!!!!!!Instance Methods'; cr.

self methodSelectors: target methodDictionary toWiki: stream.

stream nextPutAll: '!!!!!!Class Methods'; cr.

self methodSelectors: target class methodDictionary toWiki: stream.
 ! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 04:31'!
pageName

^target name asString

 ! !

!PRPierifySmalltalkClass methodsFor: 'page items' stamp: 'kph 9/15/2006 16:44'!
parentPath

^ ( target allSuperclasses reverse) joinWith: $/
 
"
(self target: ProtoObject) parentPath
(self target: String) parentPath
"! !

!PRPierifyToPage methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 00:09'!
into: aScripter
 
^ aScripter atPath: (self parentPath) addPage: (self pageName) contents: (self pageContents)

! !

!PRPierifyToPage methodsFor: 'page items' stamp: 'kph 9/15/2006 00:09'!
pageContents
	^ String streamContents: [ :stream | self pageContentsOn: stream ].! !

!PRPierifyToPage methodsFor: 'page items' stamp: 'kph 9/15/2006 00:11'!
pageContentsOn: stream

^ self subclassResponsibility! !

!PRPierifyToPage methodsFor: 'page items' stamp: 'kph 9/15/2006 00:11'!
pageName

^ self subclassResponsibility! !

!PRPierifyToPage methodsFor: 'page items' stamp: 'kph 9/15/2006 00:12'!
parentPath

^ self subclassResponsibility! !

!PRPierifyToPage methodsFor: 'as yet unclassified' stamp: 'kph 9/15/2006 00:04'!
setTarget: aDataSource

target := aDataSource! !

!String methodsFor: '*pier-scriptingapi' stamp: 'kph 9/15/2006 00:32'!
join: aCollection

	^ String streamContents: [ :stream |
		aCollection
			do: [ :each | stream nextPutAll: each ]
			separatedBy: [ stream nextPutAll: self ] ]! !
